from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend
from django.db.models import Q

from account.models import EmailAddress
from account.utils import get_user_lookup_kwargs


class UsernameAuthenticationBackend(ModelBackend):

    def authenticate(self, request, username=None, password=None, **kwargs):
        if username is None or password is None:
            return None

        User = get_user_model()
        try:
            lookup_kwargs = get_user_lookup_kwargs({
                "{username}__iexact": username
            })
            user = User.objects.get(**lookup_kwargs)
        except User.DoesNotExist:
            return None

        if user.check_password(password):
            return user


class EmailAuthenticationBackend(ModelBackend):

    def authenticate(self, request, username=None, password=None, **kwargs):
        qs = EmailAddress.objects.filter(Q(primary=True) | Q(verified=True))

        if username is None or password is None:
            return None

        try:
            email_address = qs.get(email__iexact=username)
        except EmailAddress.DoesNotExist:
            return None

        user = email_address.user
        if user.check_password(password):
            return user



TIMEZONES = [
    ("Africa/Abidjan", "Africa/Abidjan"),
    ("Africa/Accra", "Africa/Accra"),
    ("Africa/Addis_Ababa", "Africa/Addis_Ababa"),
    ("Africa/Algiers", "Africa/Algiers"),
    ("Africa/Asmara", "Africa/Asmara"),
    ("Africa/Asmera", "Africa/Asmera"),
    ("Africa/Bamako", "Africa/Bamako"),
    ("Africa/Bangui", "Africa/Bangui"),
    ("Africa/Banjul", "Africa/Banjul"),
    ("Africa/Bissau", "Africa/Bissau"),
    ("Africa/Blantyre", "Africa/Blantyre"),
    ("Africa/Brazzaville", "Africa/Brazzaville"),
    ("Africa/Bujumbura", "Africa/Bujumbura"),
    ("Africa/Cairo", "Africa/Cairo"),
    ("Africa/Casablanca", "Africa/Casablanca"),
    ("Africa/Ceuta", "Africa/Ceuta"),
    ("Africa/Conakry", "Africa/Conakry"),
    ("Africa/Dakar", "Africa/Dakar"),
    ("Africa/Dar_es_Salaam", "Africa/Dar_es_Salaam"),
    ("Africa/Djibouti", "Africa/Djibouti"),
    ("Africa/Douala", "Africa/Douala"),
    ("Africa/El_Aaiun", "Africa/El_Aaiun"),
    ("Africa/Freetown", "Africa/Freetown"),
    ("Africa/Gaborone", "Africa/Gaborone"),
    ("Africa/Harare", "Africa/Harare"),
    ("Africa/Johannesburg", "Africa/Johannesburg"),
    ("Africa/Juba", "Africa/Juba"),
    ("Africa/Kampala", "Africa/Kampala"),
    ("Africa/Khartoum", "Africa/Khartoum"),
    ("Africa/Kigali", "Africa/Kigali"),
    ("Africa/Kinshasa", "Africa/Kinshasa"),
    ("Africa/Lagos", "Africa/Lagos"),
    ("Africa/Libreville", "Africa/Libreville"),
    ("Africa/Lome", "Africa/Lome"),
    ("Africa/Luanda", "Africa/Luanda"),
    ("Africa/Lubumbashi", "Africa/Lubumbashi"),
    ("Africa/Lusaka", "Africa/Lusaka"),
    ("Africa/Malabo", "Africa/Malabo"),
    ("Africa/Maputo", "Africa/Maputo"),
    ("Africa/Maseru", "Africa/Maseru"),
    ("Africa/Mbabane", "Africa/Mbabane"),
    ("Africa/Mogadishu", "Africa/Mogadishu"),
    ("Africa/Monrovia", "Africa/Monrovia"),
    ("Africa/Nairobi", "Africa/Nairobi"),
    ("Africa/Ndjamena", "Africa/Ndjamena"),
    ("Africa/Niamey", "Africa/Niamey"),
    ("Africa/Nouakchott", "Africa/Nouakchott"),
    ("Africa/Ouagadougou", "Africa/Ouagadougou"),
    ("Africa/Porto-Novo", "Africa/Porto-Novo"),
    ("Africa/Sao_Tome", "Africa/Sao_Tome"),
    ("Africa/Timbuktu", "Africa/Timbuktu"),
    ("Africa/Tripoli", "Africa/Tripoli"),
    ("Africa/Tunis", "Africa/Tunis"),
    ("Africa/Windhoek", "Africa/Windhoek"),
    ("America/Adak", "America/Adak"),
    ("America/Anchorage", "America/Anchorage"),
    ("America/Anguilla", "America/Anguilla"),
    ("America/Antigua", "America/Antigua"),
    ("America/Araguaina", "America/Araguaina"),
    ("America/Argentina/Buenos_Aires", "America/Argentina/Buenos_Aires"),
    ("America/Argentina/Catamarca", "America/Argentina/Catamarca"),
    ("America/Argentina/ComodRivadavia", "America/Argentina/ComodRivadavia"),
    ("America/Argentina/Cordoba", "America/Argentina/Cordoba"),
    ("America/Argentina/Jujuy", "America/Argentina/Jujuy"),
    ("America/Argentina/La_Rioja", "America/Argentina/La_Rioja"),
    ("America/Argentina/Mendoza", "America/Argentina/Mendoza"),
    ("America/Argentina/Rio_Gallegos", "America/Argentina/Rio_Gallegos"),
    ("America/Argentina/Salta", "America/Argentina/Salta"),
    ("America/Argentina/San_Juan", "America/Argentina/San_Juan"),
    ("America/Argentina/San_Luis", "America/Argentina/San_Luis"),
    ("America/Argentina/Tucuman", "America/Argentina/Tucuman"),
    ("America/Argentina/Ushuaia", "America/Argentina/Ushuaia"),
    ("America/Aruba", "America/Aruba"),
    ("America/Asuncion", "America/Asuncion"),
    ("America/Atikokan", "America/Atikokan"),
    ("America/Atka", "America/Atka"),
    ("America/Bahia", "America/Bahia"),
    ("America/Bahia_Banderas", "America/Bahia_Banderas"),
    ("America/Barbados", "America/Barbados"),
    ("America/Belem", "America/Belem"),
    ("America/Belize", "America/Belize"),
    ("America/Blanc-Sablon", "America/Blanc-Sablon"),
    ("America/Boa_Vista", "America/Boa_Vista"),
    ("America/Bogota", "America/Bogota"),
    ("America/Boise", "America/Boise"),
    ("America/Buenos_Aires", "America/Buenos_Aires"),
    ("America/Cambridge_Bay", "America/Cambridge_Bay"),
    ("America/Campo_Grande", "America/Campo_Grande"),
    ("America/Cancun", "America/Cancun"),
    ("America/Caracas", "America/Caracas"),
    ("America/Catamarca", "America/Catamarca"),
    ("America/Cayenne", "America/Cayenne"),
    ("America/Cayman", "America/Cayman"),
    ("America/Chicago", "America/Chicago"),
    ("America/Chihuahua", "America/Chihuahua"),
    ("America/Coral_Harbour", "America/Coral_Harbour"),
    ("America/Cordoba", "America/Cordoba"),
    ("America/Costa_Rica", "America/Costa_Rica"),
    ("America/Creston", "America/Creston"),
    ("America/Cuiaba", "America/Cuiaba"),
    ("America/Curacao", "America/Curacao"),
    ("America/Danmarkshavn", "America/Danmarkshavn"),
    ("America/Dawson", "America/Dawson"),
    ("America/Dawson_Creek", "America/Dawson_Creek"),
    ("America/Denver", "America/Denver"),
    ("America/Detroit", "America/Detroit"),
    ("America/Dominica", "America/Dominica"),
    ("America/Edmonton", "America/Edmonton"),
    ("America/Eirunepe", "America/Eirunepe"),
    ("America/El_Salvador", "America/El_Salvador"),
    ("America/Ensenada", "America/Ensenada"),
    ("America/Fort_Wayne", "America/Fort_Wayne"),
    ("America/Fortaleza", "America/Fortaleza"),
    ("America/Glace_Bay", "America/Glace_Bay"),
    ("America/Godthab", "America/Godthab"),
    ("America/Goose_Bay", "America/Goose_Bay"),
    ("America/Grand_Turk", "America/Grand_Turk"),
    ("America/Grenada", "America/Grenada"),
    ("America/Guadeloupe", "America/Guadeloupe"),
    ("America/Guatemala", "America/Guatemala"),
    ("America/Guayaquil", "America/Guayaquil"),
    ("America/Guyana", "America/Guyana"),
    ("America/Halifax", "America/Halifax"),
    ("America/Havana", "America/Havana"),
    ("America/Hermosillo", "America/Hermosillo"),
    ("America/Indiana/Indianapolis", "America/Indiana/Indianapolis"),
    ("America/Indiana/Knox", "America/Indiana/Knox"),
    ("America/Indiana/Marengo", "America/Indiana/Marengo"),
    ("America/Indiana/Petersburg", "America/Indiana/Petersburg"),
    ("America/Indiana/Tell_City", "America/Indiana/Tell_City"),
    ("America/Indiana/Vevay", "America/Indiana/Vevay"),
    ("America/Indiana/Vincennes", "America/Indiana/Vincennes"),
    ("America/Indiana/Winamac", "America/Indiana/Winamac"),
    ("America/Indianapolis", "America/Indianapolis"),
    ("America/Inuvik", "America/Inuvik"),
    ("America/Iqaluit", "America/Iqaluit"),
    ("America/Jamaica", "America/Jamaica"),
    ("America/Jujuy", "America/Jujuy"),
    ("America/Juneau", "America/Juneau"),
    ("America/Kentucky/Louisville", "America/Kentucky/Louisville"),
    ("America/Kentucky/Monticello", "America/Kentucky/Monticello"),
    ("America/Knox_IN", "America/Knox_IN"),
    ("America/Kralendijk", "America/Kralendijk"),
    ("America/La_Paz", "America/La_Paz"),
    ("America/Lima", "America/Lima"),
    ("America/Los_Angeles", "America/Los_Angeles"),
    ("America/Louisville", "America/Louisville"),
    ("America/Lower_Princes", "America/Lower_Princes"),
    ("America/Maceio", "America/Maceio"),
    ("America/Managua", "America/Managua"),
    ("America/Manaus", "America/Manaus"),
    ("America/Marigot", "America/Marigot"),
    ("America/Martinique", "America/Martinique"),
    ("America/Matamoros", "America/Matamoros"),
    ("America/Mazatlan", "America/Mazatlan"),
    ("America/Mendoza", "America/Mendoza"),
    ("America/Menominee", "America/Menominee"),
    ("America/Merida", "America/Merida"),
    ("America/Metlakatla", "America/Metlakatla"),
    ("America/Mexico_City", "America/Mexico_City"),
    ("America/Miquelon", "America/Miquelon"),
    ("America/Moncton", "America/Moncton"),
    ("America/Monterrey", "America/Monterrey"),
    ("America/Montevideo", "America/Montevideo"),
    ("America/Montreal", "America/Montreal"),
    ("America/Montserrat", "America/Montserrat"),
    ("America/Nassau", "America/Nassau"),
    ("America/New_York", "America/New_York"),
    ("America/Nipigon", "America/Nipigon"),
    ("America/Nome", "America/Nome"),
    ("America/Noronha", "America/Noronha"),
    ("America/North_Dakota/Beulah", "America/North_Dakota/Beulah"),
    ("America/North_Dakota/Center", "America/North_Dakota/Center"),
    ("America/North_Dakota/New_Salem", "America/North_Dakota/New_Salem"),
    ("America/Ojinaga", "America/Ojinaga"),
    ("America/Panama", "America/Panama"),
    ("America/Pangnirtung", "America/Pangnirtung"),
    ("America/Paramaribo", "America/Paramaribo"),
    ("America/Phoenix", "America/Phoenix"),
    ("America/Port-au-Prince", "America/Port-au-Prince"),
    ("America/Port_of_Spain", "America/Port_of_Spain"),
    ("America/Porto_Acre", "America/Porto_Acre"),
    ("America/Porto_Velho", "America/Porto_Velho"),
    ("America/Puerto_Rico", "America/Puerto_Rico"),
    ("America/Rainy_River", "America/Rainy_River"),
    ("America/Rankin_Inlet", "America/Rankin_Inlet"),
    ("America/Recife", "America/Recife"),
    ("America/Regina", "America/Regina"),
    ("America/Resolute", "America/Resolute"),
    ("America/Rio_Branco", "America/Rio_Branco"),
    ("America/Rosario", "America/Rosario"),
    ("America/Santa_Isabel", "America/Santa_Isabel"),
    ("America/Santarem", "America/Santarem"),
    ("America/Santiago", "America/Santiago"),
    ("America/Santo_Domingo", "America/Santo_Domingo"),
    ("America/Sao_Paulo", "America/Sao_Paulo"),
    ("America/Scoresbysund", "America/Scoresbysund"),
    ("America/Shiprock", "America/Shiprock"),
    ("America/Sitka", "America/Sitka"),
    ("America/St_Barthelemy", "America/St_Barthelemy"),
    ("America/St_Johns", "America/St_Johns"),
    ("America/St_Kitts", "America/St_Kitts"),
    ("America/St_Lucia", "America/St_Lucia"),
    ("America/St_Thomas", "America/St_Thomas"),
    ("America/St_Vincent", "America/St_Vincent"),
    ("America/Swift_Current", "America/Swift_Current"),
    ("America/Tegucigalpa", "America/Tegucigalpa"),
    ("America/Thule", "America/Thule"),
    ("America/Thunder_Bay", "America/Thunder_Bay"),
    ("America/Tijuana", "America/Tijuana"),
    ("America/Toronto", "America/Toronto"),
    ("America/Tortola", "America/Tortola"),
    ("America/Vancouver", "America/Vancouver"),
    ("America/Virgin", "America/Virgin"),
    ("America/Whitehorse", "America/Whitehorse"),
    ("America/Winnipeg", "America/Winnipeg"),
    ("America/Yakutat", "America/Yakutat"),
    ("America/Yellowknife", "America/Yellowknife"),
    ("Antarctica/Casey", "Antarctica/Casey"),
    ("Antarctica/Davis", "Antarctica/Davis"),
    ("Antarctica/DumontDUrville", "Antarctica/DumontDUrville"),
    ("Antarctica/Macquarie", "Antarctica/Macquarie"),
    ("Antarctica/Mawson", "Antarctica/Mawson"),
    ("Antarctica/McMurdo", "Antarctica/McMurdo"),
    ("Antarctica/Palmer", "Antarctica/Palmer"),
    ("Antarctica/Rothera", "Antarctica/Rothera"),
    ("Antarctica/South_Pole", "Antarctica/South_Pole"),
    ("Antarctica/Syowa", "Antarctica/Syowa"),
    ("Antarctica/Troll", "Antarctica/Troll"),
    ("Antarctica/Vostok", "Antarctica/Vostok"),
    ("Arctic/Longyearbyen", "Arctic/Longyearbyen"),
    ("Asia/Aden", "Asia/Aden"),
    ("Asia/Almaty", "Asia/Almaty"),
    ("Asia/Amman", "Asia/Amman"),
    ("Asia/Anadyr", "Asia/Anadyr"),
    ("Asia/Aqtau", "Asia/Aqtau"),
    ("Asia/Aqtobe", "Asia/Aqtobe"),
    ("Asia/Ashgabat", "Asia/Ashgabat"),
    ("Asia/Ashkhabad", "Asia/Ashkhabad"),
    ("Asia/Baghdad", "Asia/Baghdad"),
    ("Asia/Bahrain", "Asia/Bahrain"),
    ("Asia/Baku", "Asia/Baku"),
    ("Asia/Bangkok", "Asia/Bangkok"),
    ("Asia/Beirut", "Asia/Beirut"),
    ("Asia/Bishkek", "Asia/Bishkek"),
    ("Asia/Brunei", "Asia/Brunei"),
    ("Asia/Calcutta", "Asia/Calcutta"),
    ("Asia/Chita", "Asia/Chita"),
    ("Asia/Choibalsan", "Asia/Choibalsan"),
    ("Asia/Chongqing", "Asia/Chongqing"),
    ("Asia/Chungking", "Asia/Chungking"),
    ("Asia/Colombo", "Asia/Colombo"),
    ("Asia/Dacca", "Asia/Dacca"),
    ("Asia/Damascus", "Asia/Damascus"),
    ("Asia/Dhaka", "Asia/Dhaka"),
    ("Asia/Dili", "Asia/Dili"),
    ("Asia/Dubai", "Asia/Dubai"),
    ("Asia/Dushanbe", "Asia/Dushanbe"),
    ("Asia/Gaza", "Asia/Gaza"),
    ("Asia/Harbin", "Asia/Harbin"),
    ("Asia/Hebron", "Asia/Hebron"),
    ("Asia/Ho_Chi_Minh", "Asia/Ho_Chi_Minh"),
    ("Asia/Hong_Kong", "Asia/Hong_Kong"),
    ("Asia/Hovd", "Asia/Hovd"),
    ("Asia/Irkutsk", "Asia/Irkutsk"),
    ("Asia/Istanbul", "Asia/Istanbul"),
    ("Asia/Jakarta", "Asia/Jakarta"),
    ("Asia/Jayapura", "Asia/Jayapura"),
    ("Asia/Jerusalem", "Asia/Jerusalem"),
    ("Asia/Kabul", "Asia/Kabul"),
    ("Asia/Kamchatka", "Asia/Kamchatka"),
    ("Asia/Karachi", "Asia/Karachi"),
    ("Asia/Kashgar", "Asia/Kashgar"),
    ("Asia/Kathmandu", "Asia/Kathmandu"),
    ("Asia/Katmandu", "Asia/Katmandu"),
    ("Asia/Khandyga", "Asia/Khandyga"),
    ("Asia/Kolkata", "Asia/Kolkata"),
    ("Asia/Krasnoyarsk", "Asia/Krasnoyarsk"),
    ("Asia/Kuala_Lumpur", "Asia/Kuala_Lumpur"),
    ("Asia/Kuching", "Asia/Kuching"),
    ("Asia/Kuwait", "Asia/Kuwait"),
    ("Asia/Macao", "Asia/Macao"),
    ("Asia/Macau", "Asia/Macau"),
    ("Asia/Magadan", "Asia/Magadan"),
    ("Asia/Makassar", "Asia/Makassar"),
    ("Asia/Manila", "Asia/Manila"),
    ("Asia/Muscat", "Asia/Muscat"),
    ("Asia/Nicosia", "Asia/Nicosia"),
    ("Asia/Novokuznetsk", "Asia/Novokuznetsk"),
    ("Asia/Novosibirsk", "Asia/Novosibirsk"),
    ("Asia/Omsk", "Asia/Omsk"),
    ("Asia/Oral", "Asia/Oral"),
    ("Asia/Phnom_Penh", "Asia/Phnom_Penh"),
    ("Asia/Pontianak", "Asia/Pontianak"),
    ("Asia/Pyongyang", "Asia/Pyongyang"),
    ("Asia/Qatar", "Asia/Qatar"),
    ("Asia/Qyzylorda", "Asia/Qyzylorda"),
    ("Asia/Rangoon", "Asia/Rangoon"),
    ("Asia/Riyadh", "Asia/Riyadh"),
    ("Asia/Saigon", "Asia/Saigon"),
    ("Asia/Sakhalin", "Asia/Sakhalin"),
    ("Asia/Samarkand", "Asia/Samarkand"),
    ("Asia/Seoul", "Asia/Seoul"),
    ("Asia/Shanghai", "Asia/Shanghai"),
    ("Asia/Singapore", "Asia/Singapore"),
    ("Asia/Srednekolymsk", "Asia/Srednekolymsk"),
    ("Asia/Taipei", "Asia/Taipei"),
    ("Asia/Tashkent", "Asia/Tashkent"),
    ("Asia/Tbilisi", "Asia/Tbilisi"),
    ("Asia/Tehran", "Asia/Tehran"),
    ("Asia/Tel_Aviv", "Asia/Tel_Aviv"),
    ("Asia/Thimbu", "Asia/Thimbu"),
    ("Asia/Thimphu", "Asia/Thimphu"),
    ("Asia/Tokyo", "Asia/Tokyo"),
    ("Asia/Ujung_Pandang", "Asia/Ujung_Pandang"),
    ("Asia/Ulaanbaatar", "Asia/Ulaanbaatar"),
    ("Asia/Ulan_Bator", "Asia/Ulan_Bator"),
    ("Asia/Urumqi", "Asia/Urumqi"),
    ("Asia/Ust-Nera", "Asia/Ust-Nera"),
    ("Asia/Vientiane", "Asia/Vientiane"),
    ("Asia/Vladivostok", "Asia/Vladivostok"),
    ("Asia/Yakutsk", "Asia/Yakutsk"),
    ("Asia/Yekaterinburg", "Asia/Yekaterinburg"),
    ("Asia/Yerevan", "Asia/Yerevan"),
    ("Atlantic/Azores", "Atlantic/Azores"),
    ("Atlantic/Bermuda", "Atlantic/Bermuda"),
    ("Atlantic/Canary", "Atlantic/Canary"),
    ("Atlantic/Cape_Verde", "Atlantic/Cape_Verde"),
    ("Atlantic/Faeroe", "Atlantic/Faeroe"),
    ("Atlantic/Faroe", "Atlantic/Faroe"),
    ("Atlantic/Jan_Mayen", "Atlantic/Jan_Mayen"),
    ("Atlantic/Madeira", "Atlantic/Madeira"),
    ("Atlantic/Reykjavik", "Atlantic/Reykjavik"),
    ("Atlantic/South_Georgia", "Atlantic/South_Georgia"),
    ("Atlantic/St_Helena", "Atlantic/St_Helena"),
    ("Atlantic/Stanley", "Atlantic/Stanley"),
    ("Australia/ACT", "Australia/ACT"),
    ("Australia/Adelaide", "Australia/Adelaide"),
    ("Australia/Brisbane", "Australia/Brisbane"),
    ("Australia/Broken_Hill", "Australia/Broken_Hill"),
    ("Australia/Canberra", "Australia/Canberra"),
    ("Australia/Currie", "Australia/Currie"),
    ("Australia/Darwin", "Australia/Darwin"),
    ("Australia/Eucla", "Australia/Eucla"),
    ("Australia/Hobart", "Australia/Hobart"),
    ("Australia/LHI", "Australia/LHI"),
    ("Australia/Lindeman", "Australia/Lindeman"),
    ("Australia/Lord_Howe", "Australia/Lord_Howe"),
    ("Australia/Melbourne", "Australia/Melbourne"),
    ("Australia/NSW", "Australia/NSW"),
    ("Australia/North", "Australia/North"),
    ("Australia/Perth", "Australia/Perth"),
    ("Australia/Queensland", "Australia/Queensland"),
    ("Australia/South", "Australia/South"),
    ("Australia/Sydney", "Australia/Sydney"),
    ("Australia/Tasmania", "Australia/Tasmania"),
    ("Australia/Victoria", "Australia/Victoria"),
    ("Australia/West", "Australia/West"),
    ("Australia/Yancowinna", "Australia/Yancowinna"),
    ("Brazil/Acre", "Brazil/Acre"),
    ("Brazil/DeNoronha", "Brazil/DeNoronha"),
    ("Brazil/East", "Brazil/East"),
    ("Brazil/West", "Brazil/West"),
    ("CET", "CET"),
    ("CST6CDT", "CST6CDT"),
    ("Canada/Atlantic", "Canada/Atlantic"),
    ("Canada/Central", "Canada/Central"),
    ("Canada/East-Saskatchewan", "Canada/East-Saskatchewan"),
    ("Canada/Eastern", "Canada/Eastern"),
    ("Canada/Mountain", "Canada/Mountain"),
    ("Canada/Newfoundland", "Canada/Newfoundland"),
    ("Canada/Pacific", "Canada/Pacific"),
    ("Canada/Saskatchewan", "Canada/Saskatchewan"),
    ("Canada/Yukon", "Canada/Yukon"),
    ("Chile/Continental", "Chile/Continental"),
    ("Chile/EasterIsland", "Chile/EasterIsland"),
    ("Cuba", "Cuba"),
    ("EET", "EET"),
    ("EST", "EST"),
    ("EST5EDT", "EST5EDT"),
    ("Egypt", "Egypt"),
    ("Eire", "Eire"),
    ("Etc/GMT", "Etc/GMT"),
    ("Etc/GMT+0", "Etc/GMT+0"),
    ("Etc/GMT+1", "Etc/GMT+1"),
    ("Etc/GMT+10", "Etc/GMT+10"),
    ("Etc/GMT+11", "Etc/GMT+11"),
    ("Etc/GMT+12", "Etc/GMT+12"),
    ("Etc/GMT+2", "Etc/GMT+2"),
    ("Etc/GMT+3", "Etc/GMT+3"),
    ("Etc/GMT+4", "Etc/GMT+4"),
    ("Etc/GMT+5", "Etc/GMT+5"),
    ("Etc/GMT+6", "Etc/GMT+6"),
    ("Etc/GMT+7", "Etc/GMT+7"),
    ("Etc/GMT+8", "Etc/GMT+8"),
    ("Etc/GMT+9", "Etc/GMT+9"),
    ("Etc/GMT-0", "Etc/GMT-0"),
    ("Etc/GMT-1", "Etc/GMT-1"),
    ("Etc/GMT-10", "Etc/GMT-10"),
    ("Etc/GMT-11", "Etc/GMT-11"),
    ("Etc/GMT-12", "Etc/GMT-12"),
    ("Etc/GMT-13", "Etc/GMT-13"),
    ("Etc/GMT-14", "Etc/GMT-14"),
    ("Etc/GMT-2", "Etc/GMT-2"),
    ("Etc/GMT-3", "Etc/GMT-3"),
    ("Etc/GMT-4", "Etc/GMT-4"),
    ("Etc/GMT-5", "Etc/GMT-5"),
    ("Etc/GMT-6", "Etc/GMT-6"),
    ("Etc/GMT-7", "Etc/GMT-7"),
    ("Etc/GMT-8", "Etc/GMT-8"),
    ("Etc/GMT-9", "Etc/GMT-9"),
    ("Etc/GMT0", "Etc/GMT0"),
    ("Etc/Greenwich", "Etc/Greenwich"),
    ("Etc/UCT", "Etc/UCT"),
    ("Etc/UTC", "Etc/UTC"),
    ("Etc/Universal", "Etc/Universal"),
    ("Etc/Zulu", "Etc/Zulu"),
    ("Europe/Amsterdam", "Europe/Amsterdam"),
    ("Europe/Andorra", "Europe/Andorra"),
    ("Europe/Athens", "Europe/Athens"),
    ("Europe/Belfast", "Europe/Belfast"),
    ("Europe/Belgrade", "Europe/Belgrade"),
    ("Europe/Berlin", "Europe/Berlin"),
    ("Europe/Bratislava", "Europe/Bratislava"),
    ("Europe/Brussels", "Europe/Brussels"),
    ("Europe/Bucharest", "Europe/Bucharest"),
    ("Europe/Budapest", "Europe/Budapest"),
    ("Europe/Busingen", "Europe/Busingen"),
    ("Europe/Chisinau", "Europe/Chisinau"),
    ("Europe/Copenhagen", "Europe/Copenhagen"),
    ("Europe/Dublin", "Europe/Dublin"),
    ("Europe/Gibraltar", "Europe/Gibraltar"),
    ("Europe/Guernsey", "Europe/Guernsey"),
    ("Europe/Helsinki", "Europe/Helsinki"),
    ("Europe/Isle_of_Man", "Europe/Isle_of_Man"),
    ("Europe/Istanbul", "Europe/Istanbul"),
    ("Europe/Jersey", "Europe/Jersey"),
    ("Europe/Kaliningrad", "Europe/Kaliningrad"),
    ("Europe/Kiev", "Europe/Kiev"),
    ("Europe/Lisbon", "Europe/Lisbon"),
    ("Europe/Ljubljana", "Europe/Ljubljana"),
    ("Europe/London", "Europe/London"),
    ("Europe/Luxembourg", "Europe/Luxembourg"),
    ("Europe/Madrid", "Europe/Madrid"),
    ("Europe/Malta", "Europe/Malta"),
    ("Europe/Mariehamn", "Europe/Mariehamn"),
    ("Europe/Minsk", "Europe/Minsk"),
    ("Europe/Monaco", "Europe/Monaco"),
    ("Europe/Moscow", "Europe/Moscow"),
    ("Europe/Nicosia", "Europe/Nicosia"),
    ("Europe/Oslo", "Europe/Oslo"),
    ("Europe/Paris", "Europe/Paris"),
    ("Europe/Podgorica", "Europe/Podgorica"),
    ("Europe/Prague", "Europe/Prague"),
    ("Europe/Riga", "Europe/Riga"),
    ("Europe/Rome", "Europe/Rome"),
    ("Europe/Samara", "Europe/Samara"),
    ("Europe/San_Marino", "Europe/San_Marino"),
    ("Europe/Sarajevo", "Europe/Sarajevo"),
    ("Europe/Simferopol", "Europe/Simferopol"),
    ("Europe/Skopje", "Europe/Skopje"),
    ("Europe/Sofia", "Europe/Sofia"),
    ("Europe/Stockholm", "Europe/Stockholm"),
    ("Europe/Tallinn", "Europe/Tallinn"),
    ("Europe/Tirane", "Europe/Tirane"),
    ("Europe/Tiraspol", "Europe/Tiraspol"),
    ("Europe/Uzhgorod", "Europe/Uzhgorod"),
    ("Europe/Vaduz", "Europe/Vaduz"),
    ("Europe/Vatican", "Europe/Vatican"),
    ("Europe/Vienna", "Europe/Vienna"),
    ("Europe/Vilnius", "Europe/Vilnius"),
    ("Europe/Volgograd", "Europe/Volgograd"),
    ("Europe/Warsaw", "Europe/Warsaw"),
    ("Europe/Zagreb", "Europe/Zagreb"),
    ("Europe/Zaporozhye", "Europe/Zaporozhye"),
    ("Europe/Zurich", "Europe/Zurich"),
    ("GB", "GB"),
    ("GB-Eire", "GB-Eire"),
    ("GMT", "GMT"),
    ("GMT+0", "GMT+0"),
    ("GMT-0", "GMT-0"),
    ("GMT0", "GMT0"),
    ("Greenwich", "Greenwich"),
    ("HST", "HST"),
    ("Hongkong", "Hongkong"),
    ("Iceland", "Iceland"),
    ("Indian/Antananarivo", "Indian/Antananarivo"),
    ("Indian/Chagos", "Indian/Chagos"),
    ("Indian/Christmas", "Indian/Christmas"),
    ("Indian/Cocos", "Indian/Cocos"),
    ("Indian/Comoro", "Indian/Comoro"),
    ("Indian/Kerguelen", "Indian/Kerguelen"),
    ("Indian/Mahe", "Indian/Mahe"),
    ("Indian/Maldives", "Indian/Maldives"),
    ("Indian/Mauritius", "Indian/Mauritius"),
    ("Indian/Mayotte", "Indian/Mayotte"),
    ("Indian/Reunion", "Indian/Reunion"),
    ("Iran", "Iran"),
    ("Israel", "Israel"),
    ("Jamaica", "Jamaica"),
    ("Japan", "Japan"),
    ("Kwajalein", "Kwajalein"),
    ("Libya", "Libya"),
    ("MET", "MET"),
    ("MST", "MST"),
    ("MST7MDT", "MST7MDT"),
    ("Mexico/BajaNorte", "Mexico/BajaNorte"),
    ("Mexico/BajaSur", "Mexico/BajaSur"),
    ("Mexico/General", "Mexico/General"),
    ("NZ", "NZ"),
    ("NZ-CHAT", "NZ-CHAT"),
    ("Navajo", "Navajo"),
    ("PRC", "PRC"),
    ("PST8PDT", "PST8PDT"),
    ("Pacific/Apia", "Pacific/Apia"),
    ("Pacific/Auckland", "Pacific/Auckland"),
    ("Pacific/Bougainville", "Pacific/Bougainville"),
    ("Pacific/Chatham", "Pacific/Chatham"),
    ("Pacific/Chuuk", "Pacific/Chuuk"),
    ("Pacific/Easter", "Pacific/Easter"),
    ("Pacific/Efate", "Pacific/Efate"),
    ("Pacific/Enderbury", "Pacific/Enderbury"),
    ("Pacific/Fakaofo", "Pacific/Fakaofo"),
    ("Pacific/Fiji", "Pacific/Fiji"),
    ("Pacific/Funafuti", "Pacific/Funafuti"),
    ("Pacific/Galapagos", "Pacific/Galapagos"),
    ("Pacific/Gambier", "Pacific/Gambier"),
    ("Pacific/Guadalcanal", "Pacific/Guadalcanal"),
    ("Pacific/Guam", "Pacific/Guam"),
    ("Pacific/Honolulu", "Pacific/Honolulu"),
    ("Pacific/Johnston", "Pacific/Johnston"),
    ("Pacific/Kiritimati", "Pacific/Kiritimati"),
    ("Pacific/Kosrae", "Pacific/Kosrae"),
    ("Pacific/Kwajalein", "Pacific/Kwajalein"),
    ("Pacific/Majuro", "Pacific/Majuro"),
    ("Pacific/Marquesas", "Pacific/Marquesas"),
    ("Pacific/Midway", "Pacific/Midway"),
    ("Pacific/Nauru", "Pacific/Nauru"),
    ("Pacific/Niue", "Pacific/Niue"),
    ("Pacific/Norfolk", "Pacific/Norfolk"),
    ("Pacific/Noumea", "Pacific/Noumea"),
    ("Pacific/Pago_Pago", "Pacific/Pago_Pago"),
    ("Pacific/Palau", "Pacific/Palau"),
    ("Pacific/Pitcairn", "Pacific/Pitcairn"),
    ("Pacific/Pohnpei", "Pacific/Pohnpei"),
    ("Pacific/Ponape", "Pacific/Ponape"),
    ("Pacific/Port_Moresby", "Pacific/Port_Moresby"),
    ("Pacific/Rarotonga", "Pacific/Rarotonga"),
    ("Pacific/Saipan", "Pacific/Saipan"),
    ("Pacific/Samoa", "Pacific/Samoa"),
    ("Pacific/Tahiti", "Pacific/Tahiti"),
    ("Pacific/Tarawa", "Pacific/Tarawa"),
    ("Pacific/Tongatapu", "Pacific/Tongatapu"),
    ("Pacific/Truk", "Pacific/Truk"),
    ("Pacific/Wake", "Pacific/Wake"),
    ("Pacific/Wallis", "Pacific/Wallis"),
    ("Pacific/Yap", "Pacific/Yap"),
    ("Poland", "Poland"),
    ("Portugal", "Portugal"),
    ("ROC", "ROC"),
    ("ROK", "ROK"),
    ("Singapore", "Singapore"),
    ("Turkey", "Turkey"),
    ("UCT", "UCT"),
    ("US/Alaska", "US/Alaska"),
    ("US/Aleutian", "US/Aleutian"),
    ("US/Arizona", "US/Arizona"),
    ("US/Central", "US/Central"),
    ("US/East-Indiana", "US/East-Indiana"),
    ("US/Eastern", "US/Eastern"),
    ("US/Hawaii", "US/Hawaii"),
    ("US/Indiana-Starke", "US/Indiana-Starke"),
    ("US/Michigan", "US/Michigan"),
    ("US/Mountain", "US/Mountain"),
    ("US/Pacific", "US/Pacific"),
    ("US/Pacific-New", "US/Pacific-New"),
    ("US/Samoa", "US/Samoa"),
    ("UTC", "UTC"),
    ("Universal", "Universal"),
    ("W-SU", "W-SU"),
    ("WET", "WET"),
    ("Zulu", "Zulu")
]



import re
from collections import OrderedDict

from django import forms
from django.contrib import auth
from django.contrib.auth import get_user_model
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _

from account.conf import settings
from account.hooks import hookset
from account.models import EmailAddress
from account.utils import get_user_lookup_kwargs

alnum_re = re.compile(r"^\w+$")


class PasswordField(forms.CharField):

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("widget", forms.PasswordInput(render_value=False))
        self.strip = kwargs.pop("strip", True)
        super(PasswordField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if value in self.empty_values:
            return ""
        value = force_str(value)
        if self.strip:
            value = value.strip()
        return value


class SignupForm(forms.Form):

    username = forms.CharField(
        label=_("Username"),
        max_length=30,
        widget=forms.TextInput(),
        required=True
    )
    email = forms.EmailField(
        label=_("Email"),
        widget=forms.TextInput(), required=True
    )
    password = PasswordField(
        label=_("Password"),
        strip=settings.ACCOUNT_PASSWORD_STRIP,
    )
    password_confirm = PasswordField(
        label=_("Password (again)"),
        strip=settings.ACCOUNT_PASSWORD_STRIP,
    )
    code = forms.CharField(
        max_length=64,
        required=False,
        widget=forms.HiddenInput()
    )

    def clean_username(self):
        if not alnum_re.search(self.cleaned_data["username"]):
            raise forms.ValidationError(_("Usernames can only contain letters, numbers and underscores."))
        User = get_user_model()
        lookup_kwargs = get_user_lookup_kwargs({
            "{username}__iexact": self.cleaned_data["username"]
        })
        qs = User.objects.filter(**lookup_kwargs)
        if not qs.exists():
            return self.cleaned_data["username"]
        raise forms.ValidationError(_("This username is already taken. Please choose another."))

    def clean_email(self):
        value = self.cleaned_data["email"]
        qs = EmailAddress.objects.filter(email__iexact=value)
        if not qs.exists() or not settings.ACCOUNT_EMAIL_UNIQUE:
            return value
        raise forms.ValidationError(_("A user is registered with this email address."))

    def clean(self):
        if "password" in self.cleaned_data and "password_confirm" in self.cleaned_data:
            if self.cleaned_data["password"] != self.cleaned_data["password_confirm"]:
                raise forms.ValidationError(_("You must type the same password each time."))
        return self.cleaned_data


class LoginForm(forms.Form):

    password = PasswordField(
        label=_("Password"),
        strip=settings.ACCOUNT_PASSWORD_STRIP,
    )
    remember = forms.BooleanField(
        label=_("Remember Me"),
        required=False
    )
    user = None

    def clean(self):
        if self._errors:
            return
        user = auth.authenticate(**self.user_credentials())
        if user:
            if user.is_active:
                self.user = user
            else:
                raise forms.ValidationError(_("This account is inactive."))
        else:
            raise forms.ValidationError(self.authentication_fail_message)
        return self.cleaned_data

    def user_credentials(self):
        return hookset.get_user_credentials(self, self.identifier_field)


class LoginUsernameForm(LoginForm):

    username = forms.CharField(label=_("Username"), max_length=30)
    authentication_fail_message = _("The username and/or password you specified are not correct.")
    identifier_field = "username"

    def __init__(self, *args, **kwargs):
        super(LoginUsernameForm, self).__init__(*args, **kwargs)
        field_order = ["username", "password", "remember"]
        if hasattr(self.fields, "keyOrder"):
            self.fields.keyOrder = field_order
        else:
            self.fields = OrderedDict((k, self.fields[k]) for k in field_order)


class LoginEmailForm(LoginForm):

    email = forms.EmailField(label=_("Email"))
    authentication_fail_message = _("The email address and/or password you specified are not correct.")
    identifier_field = "email"

    def __init__(self, *args, **kwargs):
        super(LoginEmailForm, self).__init__(*args, **kwargs)
        field_order = ["email", "password", "remember"]
        if hasattr(self.fields, "keyOrder"):
            self.fields.keyOrder = field_order
        else:
            self.fields = OrderedDict((k, self.fields[k]) for k in field_order)


class ChangePasswordForm(forms.Form):

    password_current = forms.CharField(
        label=_("Current Password"),
        widget=forms.PasswordInput(render_value=False)
    )
    password_new = forms.CharField(
        label=_("New Password"),
        widget=forms.PasswordInput(render_value=False)
    )
    password_new_confirm = forms.CharField(
        label=_("New Password (again)"),
        widget=forms.PasswordInput(render_value=False)
    )

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop("user")
        super(ChangePasswordForm, self).__init__(*args, **kwargs)

    def clean_password_current(self):
        if not self.user.check_password(self.cleaned_data.get("password_current")):
            raise forms.ValidationError(_("Please type your current password."))
        return self.cleaned_data["password_current"]

    def clean_password_new_confirm(self):
        if "password_new" in self.cleaned_data and "password_new_confirm" in self.cleaned_data:
            if self.cleaned_data["password_new"] != self.cleaned_data["password_new_confirm"]:
                raise forms.ValidationError(_("You must type the same password each time."))
        return self.cleaned_data["password_new_confirm"]


class PasswordResetForm(forms.Form):

    email = forms.EmailField(label=_("Email"), required=True)

    def clean_email(self):
        value = self.cleaned_data["email"]
        if not EmailAddress.objects.filter(email__iexact=value).exists():
            raise forms.ValidationError(_("Email address can not be found."))
        return value


class PasswordResetTokenForm(forms.Form):

    password = forms.CharField(
        label=_("New Password"),
        widget=forms.PasswordInput(render_value=False)
    )
    password_confirm = forms.CharField(
        label=_("New Password (again)"),
        widget=forms.PasswordInput(render_value=False)
    )

    def clean_password_confirm(self):
        if "password" in self.cleaned_data and "password_confirm" in self.cleaned_data:
            if self.cleaned_data["password"] != self.cleaned_data["password_confirm"]:
                raise forms.ValidationError(_("You must type the same password each time."))
        return self.cleaned_data["password_confirm"]


class SettingsForm(forms.Form):

    email = forms.EmailField(label=_("Email"), required=True)
    timezone = forms.ChoiceField(
        label=_("Timezone"),
        choices=[("", "---------")] + settings.ACCOUNT_TIMEZONES,
        required=False
    )
    if settings.USE_I18N:
        language = forms.ChoiceField(
            label=_("Language"),
            choices=settings.ACCOUNT_LANGUAGES,
            required=False
        )

    def clean_email(self):
        value = self.cleaned_data["email"]
        if self.initial.get("email") == value:
            return value
        qs = EmailAddress.objects.filter(email__iexact=value)
        if not qs.exists() or not settings.ACCOUNT_EMAIL_UNIQUE:
            return value
        raise forms.ValidationError(_("A user is registered with this email address."))



import datetime
import functools
from urllib.parse import urlparse, urlunparse

from django.contrib.auth import get_user_model
from django.core.exceptions import SuspiciousOperation
from django.http import HttpResponseRedirect, QueryDict
from django.urls import NoReverseMatch, reverse
from django.utils.encoding import force_str

import pytz
from account.conf import settings

from .models import PasswordHistory


def get_user_lookup_kwargs(kwargs):
    result = {}
    username_field = getattr(get_user_model(), "USERNAME_FIELD", "username")
    for key, value in kwargs.items():
        result[key.format(username=username_field)] = value
    return result


def default_redirect(request, fallback_url, **kwargs):
    redirect_field_name = kwargs.get("redirect_field_name", "next")
    next_url = request.POST.get(redirect_field_name, request.GET.get(redirect_field_name))
    if not next_url:
        # try the session if available
        if hasattr(request, "session"):
            session_key_value = kwargs.get("session_key_value", "redirect_to")
            if session_key_value in request.session:
                next_url = request.session[session_key_value]
                del request.session[session_key_value]
    is_safe = functools.partial(
        ensure_safe_url,
        allowed_protocols=kwargs.get("allowed_protocols"),
        allowed_host=request.get_host()
    )
    if next_url and is_safe(next_url):
        return next_url
    else:
        try:
            fallback_url = reverse(fallback_url)
        except NoReverseMatch:
            if callable(fallback_url):
                raise
            if "/" not in fallback_url and "." not in fallback_url:
                raise
        # assert the fallback URL is safe to return to caller. if it is
        # determined unsafe then raise an exception as the fallback value comes
        # from the a source the developer choose.
        is_safe(fallback_url, raise_on_fail=True)
        return fallback_url


def user_display(user):
    return settings.ACCOUNT_USER_DISPLAY(user)


def ensure_safe_url(url, allowed_protocols=None, allowed_host=None, raise_on_fail=False):
    if allowed_protocols is None:
        allowed_protocols = ["http", "https"]
    parsed = urlparse(url)
    # perform security checks to ensure no malicious intent
    # (i.e., an XSS attack with a data URL)
    safe = True
    if parsed.scheme and parsed.scheme not in allowed_protocols:
        if raise_on_fail:
            raise SuspiciousOperation("Unsafe redirect to URL with protocol '{0}'".format(parsed.scheme))
        safe = False
    if allowed_host and parsed.netloc and parsed.netloc != allowed_host:
        if raise_on_fail:
            raise SuspiciousOperation("Unsafe redirect to URL not matching host '{0}'".format(allowed_host))
        safe = False
    return safe


def handle_redirect_to_login(request, **kwargs):
    login_url = kwargs.get("login_url")
    redirect_field_name = kwargs.get("redirect_field_name")
    next_url = kwargs.get("next_url")
    if login_url is None:
        login_url = settings.ACCOUNT_LOGIN_URL
    if next_url is None:
        next_url = request.get_full_path()
    try:
        login_url = reverse(login_url)
    except NoReverseMatch:
        if callable(login_url):
            raise
        if "/" not in login_url and "." not in login_url:
            raise
    url_bits = list(urlparse(force_str(login_url)))
    if redirect_field_name:
        querystring = QueryDict(url_bits[4], mutable=True)
        querystring[redirect_field_name] = next_url
        url_bits[4] = querystring.urlencode(safe="/")
    return HttpResponseRedirect(urlunparse(url_bits))


def get_form_data(form, field_name, default=None):
    if form.prefix:
        key = "-".join([form.prefix, field_name])
    else:
        key = field_name
    return form.data.get(key, default)


def check_password_expired(user):
    """
    Return True if password is expired and system is using
    password expiration, False otherwise.
    """
    if not settings.ACCOUNT_PASSWORD_USE_HISTORY:
        return False

    if hasattr(user, "password_expiry"):
        # user-specific value
        expiry = user.password_expiry.expiry
    else:
        # use global value
        expiry = settings.ACCOUNT_PASSWORD_EXPIRY

    if expiry == 0:  # zero indicates no expiration
        return False

    try:
        # get latest password info
        latest = user.password_history.latest("timestamp")
    except PasswordHistory.DoesNotExist:
        return False

    now = datetime.datetime.now(tz=pytz.UTC)
    expiration = latest.timestamp + datetime.timedelta(seconds=expiry)

    if expiration < now:
        return True
    else:
        return False



from unittest import mock

from django.http import HttpResponse
from django.test import TestCase

from account.decorators import login_required


@login_required
def mock_view(request, *args, **kwargs):
    return HttpResponse('OK', status=200)


class LoginRequiredDecoratorTestCase(TestCase):

    def test_authenticated_user_is_allowed(self):
        request = mock.MagicMock()
        request.user.is_authenticated = True
        response = mock_view(request)
        self.assertEqual(response.status_code, 200)

    def test_unauthenticated_user_gets_redirected(self):
        request = mock.MagicMock()
        request.user.is_authenticated = False
        response = mock_view(request)
        self.assertEqual(response.status_code, 302)



from django.contrib.auth import get_user_model
from django.core.management.base import LabelCommand

from account.conf import settings
from account.models import PasswordExpiry


class Command(LabelCommand):

    help = "Create user-specific password expiration period."
    label = "username"

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "-e", "--expire",
            type=int,
            nargs="?",
            default=settings.ACCOUNT_PASSWORD_EXPIRY,
            help="number of seconds until password expires"
        )

    def handle_label(self, username, **options):
        User = get_user_model()
        try:
            user = User.objects.get(username=username)
        except User.DoesNotExist:
            return 'User "{}" not found'.format(username)

        expire = options["expire"]

        # Modify existing PasswordExpiry or create new if needed.
        if not hasattr(user, "password_expiry"):
            PasswordExpiry.objects.create(user=user, expiry=expire)
        else:
            user.password_expiry.expiry = expire
            user.password_expiry.save()

        return 'User "{}" password expiration set to {} seconds'.format(username, expire)



from setuptools import find_packages, setup

setup(
    name="django-user-accounts",
    version="3.0.3",
    author="Brian Rosner",
    author_email="brosner@gmail.com",
    description="a Django user account app",
    long_description=open("README.md").read(),
    license="MIT",
    url="http://github.com/pinax/django-user-accounts",
    packages=find_packages(),
    install_requires=[
        "Django>=2.2",
        "django-appconf>=1.0.4",
        "pytz>=2020.4"
    ],
    zip_safe=False,
    package_data={
        "account": [
            "locale/*/LC_MESSAGES/*",
        ],
    },
    extras_require={
        "pytest": ["pytest", "pytest-django"]
    },
    test_suite="runtests.runtests",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.1',
        'Framework :: Django :: 3.2',
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        'Programming Language :: Python :: 3',
    ]
)



from django.db import models

from account.conf import settings


class TimeZoneField(models.CharField):

    def __init__(self, *args, **kwargs):
        defaults = {
            "max_length": 100,
            "default": "",
            "choices": settings.ACCOUNT_TIMEZONES,
            "blank": True,
        }
        defaults.update(kwargs)
        return super(TimeZoneField, self).__init__(*args, **defaults)
